/* Program written to read some fields of the 2MASS star catalog
   Saves output as a .mat file.
 
   Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
   This is free software - See file LICENSE.txt for terms and conditions.
   Written by Marcio A. A. Fialho.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#define PROGNAME "READ_2MASS"

// maximum length of the catalog section name provided to the program
#define MAX_CATSECTNAME_LEN 16

// maximum length of lines in the source file
#define MAX_LINELEN 1020

// number of fields to extract from the 2MASS star catalog
#define NUMFIELDS 5  // fields to extract: RA, dec, Jmag, Hmag, Kmag.


typedef struct 
{
	uint32_t type;
	uint32_t mrows;
	uint32_t ncols;
	uint32_t imagf;
	uint32_t namelen;
} Fmatrix;


typedef struct
{
	double RA; //right ascension
	double dec; // declination
	double J; // magnitude in J band
	double H; // magnitude in H band
	double K; // magnitude in K band
} star_entry;


// Return values:
#define SUCCESS 0
#define INCOMPLETE_LINE 1
#define UNEXPECTED_CHAR 2
#define FWRITE_ERROR 3
#define TOOLONGLINE 4
#define FGETS_ERROR 5
#define NULL_FIELD 6



/* Writes header of .mat file - terminates the program in case of failure
 * @param[in] name => name of the matrix in Matlab.
 * @param[out] dest => target file.
 */
static void write_header(const char *name, FILE *dest)
{
	Fmatrix x;
	
	x.type = 0; // little endian format, double precision, numeric matrix.
	            // use x.type = 1000 = 0x03E8 if your machine is big-endian.
	x.mrows = NUMFIELDS; // In Matlab, matrices follow column major order,
	                     // hence the number of fields (columns) become
	                     // number of rows.
	x.ncols = 0; // number of successfuly converted star entries =
	             // number columns in Matlab.
	x.imagf = 0; // no imaginary data
	x.namelen = strlen(name) + 1;
	
	// Attempts to write file header
	if (fwrite(&x, sizeof(Fmatrix), 1, dest) != 1)
	{
		fprintf(stderr, "Error writing destination file header.\n");
		exit(1);
	}
	
	// Attempts to write matrix name
	if (fwrite(name, sizeof(char), x.namelen, dest) != x.namelen)
	{
		fprintf(stderr, "Error writing matrix name in destination file.\n");
		exit(1);
	}
}


// Changes the number of entries in the header.
static void patch_dest_header(int num_entries, FILE *dest)
{
	int retval;
	
	// There are two fields before the ncols field that we want to patch in the
	// file header.
	retval = fseek(dest, 2 * sizeof(uint32_t), SEEK_SET);
	if (retval)
	{
		fprintf(stderr, "Error writing destination file header.\n");
		exit(1);
	}
	
	uint32_t aux = num_entries;
	if (fwrite(&aux, sizeof(aux), 1, dest) != 1)
	{
		fprintf(stderr, "Error writing destination file header.\n");
		exit(1);
	}
	
	retval = fseek(dest, 0, SEEK_END);
	if (retval)
	{
		fprintf(stderr, "Error writing destination file header.\n");
		exit(1);
	}
}


/* Helper function for process_line */
static int endptr_errormsg(char *endptr, char *linebuff, int linecnt)
{
	if (*endptr == 0) {
		fprintf(stderr,"Warning: Line %d is incomplete - "
			"unexpected line end while reading input field.\n", linecnt);
		fflush(stderr);
		return INCOMPLETE_LINE;
	}
	
	if ( (endptr[0] == '\\') && (endptr[1] == 'N') )
	{
		fprintf (stderr, "Warning: Null field at line %d column %d.\n",
			linecnt, endptr - linebuff + 1);
		fflush(stderr);
		return NULL_FIELD;
	}
	
	fprintf(stderr,"Warning: Unexpected character '%c' at"
		" line %d column %d.\n",
		*endptr, linecnt, endptr - linebuff + 1);
	fflush(stderr);
	return UNEXPECTED_CHAR;
}


/* Helper function for process_line */
static int skipfields(char ** auxptr, int delim, int count, int linecnt)
{
	int i;
	for (i=0; i < count; i++) 
	{
		*auxptr = strchr(*auxptr, delim); 
		if (*auxptr == NULL) {
			fprintf(stderr,
				"Warning: Line %d is incomplete - not enough fields!\n",
				linecnt);
			fflush(stderr);
			return INCOMPLETE_LINE;
		}
		(*auxptr)++;
	}
	
	return SUCCESS;
}



/** Extracts fields from a line read from the source code
 * modifies input buffer linebuff
 * returns: 0 on success or a non-zero number on errors
 */
static int process_line(char *linebuff, int linecnt, star_entry *star)
{
	int retval;
	const char delim='|';
	char *auxptr = linebuff;
	char *endptr;


	// First field - Right ascension
	star->RA = strtod(auxptr, &endptr);
	if (*endptr != delim) {
		return endptr_errormsg(endptr, linebuff, linecnt);
	}
	auxptr = endptr + 1; // advances to the next field.


	// Second field - declination
	star->dec = strtod(auxptr, &endptr);
	if (*endptr != delim) {
		return endptr_errormsg(endptr, linebuff, linecnt);
	}
	auxptr = endptr + 1; // advances to the next field.


	// Skips next four fields
	retval = skipfields(&auxptr, delim, 4, linecnt);
	if (retval)
		return retval;

	
	//Seventh field - magnitude in J band
	star->J = strtod(auxptr, &endptr);
	if (*endptr != delim) {
		return endptr_errormsg(endptr, linebuff, linecnt);
	}
	auxptr = endptr + 1; // advances to the next field.


	// Skips next three fields
	retval = skipfields(&auxptr, delim, 3, linecnt);
	if (retval)
		return retval;


	//eleventh field - magnitude in H band
	star->H = strtod(auxptr, &endptr);
	if (*endptr != delim) {
		return endptr_errormsg(endptr, linebuff, linecnt);
	}
	auxptr = endptr + 1; // advances to the next field.


	// Skips next three fields
	retval = skipfields(&auxptr, delim, 3, linecnt);
	if (retval)
		return retval;


	//Eleventh field - magnitude in H band
	star->K = strtod(auxptr, &endptr);
	if (*endptr != delim) {
		return endptr_errormsg(endptr, linebuff, linecnt);
	}
	auxptr = endptr + 1; // advances to the next field.

	return SUCCESS;
}



static int process_body(FILE *src, FILE *dest, int *saved_entries_ptr)
{
	char linebuff[MAX_LINELEN + 1];
	int linecnt = 0;
	int badlines = 0;
	int saved_entries = 0;
	star_entry star;
	
	saved_entries = 0;
	*saved_entries_ptr = saved_entries;
	while (fgets(linebuff, sizeof(linebuff), src) != NULL)
	{
		linecnt++;
		int linelen = strlen(linebuff);
		if (linelen >= MAX_LINELEN)
		{
			fprintf(stderr,"Error: Line %d is too long\n", linecnt);
			fflush(stderr);
			*saved_entries_ptr = saved_entries;
			return TOOLONGLINE;
		}
		
		int retval = process_line(linebuff, linecnt, &star);
		if (retval != 0)
		{
			badlines++;
		}
		else
		{
			//writes output to file
			if (fwrite(&star, sizeof(star), 1, dest) != 1)
			{
				perror("Error when writing target file");
				*saved_entries_ptr = saved_entries;
				return FWRITE_ERROR;
			}
			saved_entries++;
		}
	}
	
	// The previous loop should break only at the end of the file
	if (!feof(src))
	{
		fprintf(stderr, 
			"Warning: An error occurred when attempting to read line %d, "
			"causing this program to terminate prematurely."
			" Output might not be complete\n",
			linecnt + 1);
		fflush(stderr);
		return FGETS_ERROR;
	}
	
	printf("%d lines processed.\n", linecnt);
	printf("%d bad lines ignored.\n", badlines);
	printf("%d lines saved.\n", saved_entries);
	
	*saved_entries_ptr = saved_entries;
	
	return SUCCESS;
}



static void print_usage(const char *progname)
{
	printf(PROGNAME 
		": extracts some fields from the 2MASS star catalog, saving \n"
		"            results as a .mat file.\n"
		"Usage: %s <section_name> [src_file] [dest_file]\n", progname);
	printf(
		" <section_name> = catalog section name: e.g.: psc_aaa, psc_aab, etc.\n"
		" [src_file]  = optional argument with the name of the input file.\n"
		"               If not given, considered to be equal to <section_name>\n"
		"               (eg.: psc_aaa, with no extension).\n");
	printf(
		" [dest_file] = optional argument with the name of the output file.\n"
		"               If not given, considered to be <section_name>.mat\n"
		"               (eg.: psc_aaa.mat).\n");
}



static void test_num_arguments(int argc, const char* argv[])
{
	// Prints usage 
	if (argc<2)
	{
		print_usage(argv[0]);
		exit(EXIT_FAILURE);
	}
}



/** Tests the size of strings passed as input arguments to the program.
 * Terminates the program with an error message if the string is too long.
 * @param inputpar - input parameter to be tested
 * @param maxlen - maximum allowed string length
 * @param inputname - a string identificating the input parameter to be printed
 *                in the error message. It can be, for example, "section_name"
 *                or "source file name " or "dest_file name".
 */
static void check_namelen(const char* inputpar, const int maxlen,
                          const char* inputname)
{
	int namelen = strlen(inputpar);
	if (namelen > maxlen)
	{
		fprintf(stderr, "Error: Provided %s is too long.\n"
			" Cannot handle %s larger than %d bytes.\n",
			inputname, inputname, MAX_CATSECTNAME_LEN);
		exit(EXIT_FAILURE);
	}
}



/** Attempts to open a file, terminating the program in case of failure
 */
static FILE* fopenx(const char *filename, const char *mode)
{
	FILE *f;
	f = fopen(filename, mode);
	if (f == NULL)
	{
		perror(filename);
		exit(EXIT_FAILURE);
	}
	return f;
}



/** Main function - program entry point 
 */
int main(int argc, const char* argv[])
{
	// Prints usage if not enough arguments
	test_num_arguments(argc, argv);
	
	// Gets section name from input argument
	const char* section_name = argv[1];
	check_namelen(section_name, MAX_CATSECTNAME_LEN, "section_name");
	
	// Source file name
	const char* srcname = section_name;
	if (argc > 2) // two or more input arguments besides the program name
	{
		srcname = argv[2];
	}
	
	// Destination file extension to use, when target file name is derived 
	// from the section name
	const char matfile_ext[] = ".mat";
	
	// Destination file name
	char destnamebuf[MAX_CATSECTNAME_LEN + sizeof(matfile_ext)];
	const char* destname;
	if (argc < 4) // less than three input arguments (excluding program name)
	{
		strcpy(destnamebuf, section_name);
		strcat(destnamebuf, matfile_ext); // appends file extension.
		destname = destnamebuf;
	}
	else
	{
		destname = argv[3];
	}
	
	// // Debug - print arguments after processing and exits.
	// printf("%s %s %s %s\n", argv[0], section_name, srcname, destname);
	// return 1;
	
	// Opens files and do processing.
	FILE *src;
	FILE *dest;
	src = fopenx(srcname, "rb");
	dest = fopenx(destname, "wb");
	
	write_header(section_name, dest);
	int num_entries = 0;
	int retval = process_body(src, dest, &num_entries);
	if (retval != 0)
	{
		printf("There were some errors!\n");
	}
	
	fclose(src);
	
	patch_dest_header(num_entries, dest);
	fclose(dest);
	
	return 0;
}