/* Program written to read some fields of the 2MASS star catalog
   Saves output as a text file.
  
   Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
   This is free software - see file License.txt for terms and conditions.
 
   Written by Marcio A. A. Fialho.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define PROGNAME "READ_2MASS"

// maximum length of the catalog section name provided to the program
#define MAX_BASENAME_LEN 250

// maximum length of lines in the source file
#define MAX_LINELEN 1020


/** Prints the header in the output file */
static void print_header(FILE *dest, const char* basename)
{
	fprintf(dest, "%% File automatically generated by program " PROGNAME 
		" from file %s\n", basename);
	fprintf(dest, "%% Format:\n");
	fprintf(dest, "%%  RA     |  dec     |  J   |  H   |  K   |\n");
}


/** Extracts fields from a line read from the source code
 * modifies input buffer linebuff
 * returns: 0 on success or a non-zero number on errors
 */
static int process_line(char *linebuff, int linecnt, FILE *dest)
{
	char *auxptr = linebuff;
	const char delim='|';


	char *RA_text = auxptr; //first field - Right ascension
	auxptr = strchr(auxptr, delim);
	if (auxptr == NULL) {
		goto jmp_incomplete_line; //at the end of this function
	}
	unsigned int RA_text_len = auxptr - RA_text; //computes text length
	*auxptr = 0; //adds null terminator for the first field
	auxptr++; // advances to the next field.


	char *dec_text = auxptr; //second field - declination
	auxptr = strchr(auxptr, delim);
	if (auxptr == NULL) {
		goto jmp_incomplete_line; //at the end of this function
	}
	unsigned int dec_text_len = auxptr - dec_text; //computes text length
	*auxptr = 0; //adds null terminator for the second field
	auxptr++; // advances to the next field.


	int i;
	for (i=0; i < 4; i++) // Skips next four fields
	{
		auxptr = strchr(auxptr, delim); 
		if (auxptr == NULL) {
			goto jmp_incomplete_line;
		}
		auxptr++;
	}


	char *j_m_text = auxptr; //seventh field - magnitude in J band
	auxptr = strchr(auxptr, delim);
	if (auxptr == NULL) {
		goto jmp_incomplete_line;
	}
	unsigned int j_m_text_len = auxptr - j_m_text;
	*auxptr = 0;
	auxptr++;


	for (i=0; i < 3; i++) // Skips next three fields
	{
		auxptr = strchr(auxptr, delim); 
		if (auxptr == NULL) {
			goto jmp_incomplete_line;
		}
		auxptr++;
	}


	char *h_m_text = auxptr; //eleventh field - magnitude in H band
	auxptr = strchr(auxptr, delim);
	if (auxptr == NULL) {
		goto jmp_incomplete_line;
	}
	unsigned int h_m_text_len = auxptr - h_m_text;
	*auxptr = 0;
	auxptr++;


	for (i=0; i < 3; i++) // Skips next three fields
	{
		auxptr = strchr(auxptr, delim); 
		if (auxptr == NULL) {
			goto jmp_incomplete_line;
		}
		auxptr++;
	}


	char *k_m_text = auxptr; //fourteenth field - magnitude in K band
	auxptr = strchr(auxptr, delim);
	if (auxptr == NULL) {
		goto jmp_incomplete_line;
	}
	unsigned int k_m_text_len = auxptr - k_m_text;
	*auxptr = 0;
	auxptr++;


	// Checks the length of catalog fields
	if ( (RA_text_len  > 10) ||
	     (dec_text_len > 10) ||
	     (j_m_text_len >  6) ||
	     (h_m_text_len >  6) ||
	     (k_m_text_len >  6) )
	{
		return 1; // Error - invalid catalog field length
	}


	//writes output to file
	fprintf(dest, "%10s|%10s|%6s|%6s|%6s|\n", RA_text, dec_text,
		j_m_text, h_m_text, k_m_text);
	
	return 0; // normal exit:


	// In case of unexpected line ending or malformed line:
  jmp_incomplete_line:
	return 2; // Error - incomplete line.
}



static void print_body(FILE *src, FILE *dest)
{
	char linebuff[MAX_LINELEN + 1];
	int linecnt = 0;
	int badlines = 0;
	while (fgets(linebuff, sizeof(linebuff), src) != NULL)
	{
		linecnt++;
		int linelen = strlen(linebuff);
		if (linelen >= MAX_LINELEN)
		{
			fprintf(stderr,"Error at line %d: line too long\n", linecnt);
			exit(1);
		}
		
		int retval = process_line(linebuff, linecnt, dest);
		if (retval != 0)
		{
			badlines++;
		}
	}
	
	// The previous loop should break only at the end of the file
	if (!feof(src))
	{
		fprintf(stderr, 
			"Warning: An error occurred when attempting to read line %d, "
			"causing this program to abort. Output might not be complete\n",
			linecnt + 1);
		exit(1);
	}
	
	printf("%d lines processed.\n", linecnt);
	printf("%d bad lines ignored.\n", badlines);
}

int main(int argc, const char* argv[])
{
	// Prints usage 
	if (argc<2)
	{
		printf(PROGNAME ": extracts some fields from the 2MASS star catalog,\n"
		" saving results as a smaller, more manageable text file.\n"
		"Usage: %s <section_name>\n"
		" <section_name> = catalog section name: e.g.: psc_aaa,"
		" psc_aab, etc.\n",
			argv[0]);
		return 1;
	}
	
	const char* basename = argv[1];
	
	int basename_len = strlen(basename);
	if (basename_len > MAX_BASENAME_LEN)
	{
		fprintf(stderr, "Error: Provided section_name is too long.\n"
			" Cannot handle section_names larger than %d bytes.\n",
			MAX_BASENAME_LEN);
		return 1;
	}
	
	FILE *src;
	src = fopen(basename, "rb");
	if (src == NULL)
	{
		perror(basename);
		return 1;
	}
	
	// Destination file name
	char destfname[MAX_BASENAME_LEN + 6];
	memcpy(destfname, basename, basename_len);
	destfname[basename_len] = '.'; // add file extension
	destfname[basename_len + 1] = 't'; // add file extension
	destfname[basename_len + 2] = 'x'; // add file extension
	destfname[basename_len + 3] = 't'; // add file extension
	destfname[basename_len + 4] = 0; // add file extension
	
	FILE *dest;
	dest = fopen(destfname, "wb");
	if (dest == NULL)
	{
		perror(destfname);
		fclose(src);
		return 1;
	}
	
	print_header(dest, basename);
	print_body(src, dest);
	
	fclose(src);
	fclose(dest);
	
	return 0;
}