function F = calcFisherMat(V, W)
% F = calcFisherMat(V, S) computes the Fisher information matrix for a set
% of N 3D-vectors, each having a weight W(i) = 1/variance(i).
% inputs:
%   V = a 3 x N matrix where each column gives a 3D vector contributing to
%       attitude knowledge.
%   W = a 1 x N row vector where each element gives the weight
%       corresponding to each vector V(:,i) in V. These weights give the
%       information contributed by each vector = the reciprocal of the
%       variance associated with each vector.
% Note: the 3D vectors in V should be unit vectors.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Ref.1 : F.L.Markley and J.L.Crassidis. "Fundamentals of Spacecraft
%         Attitude Determination and Control". Springer. 2014.
%         ISBN 978-1-4939-0801-1.

% Checking input arguments
if size(V,1) ~= 3
    error('Array V must be a 3 x N matrix.');
end

if (size(W,1) ~= 1) || (size(W,2) ~= size(V,2))
    error(['Array W must be a row vector with width matching the' ...
        ' width of V.']);
end


% % Slow method
% N = size(W,2);
% F = zeros(3);
% for k=1:N
%     F = F + W(k) * (eye(3) - V(:,k) * (V(:,k)'));
% end


% Fast method
 % computing main diagonal
f11 = sum( W .* (1 - V(1,:) .^ 2) );
f22 = sum( W .* (1 - V(2,:) .^ 2) );
f33 = sum( W .* (1 - V(3,:) .^ 2) );
 % computing off diagonal terms
f12 = - sum( W .* V(1,:) .* V(2,:) );
f13 = - sum( W .* V(1,:) .* V(3,:) );
f23 = - sum( W .* V(2,:) .* V(3,:) );
f21 = - sum( W .* V(2,:) .* V(1,:) );
f31 = - sum( W .* V(3,:) .* V(1,:) );
f32 = - sum( W .* V(3,:) .* V(2,:) );
 % assembling the matrix
F = [f11 f12 f13;...
     f21 f22 f23;...
     f31 f32 f33];
end

% Notice: if V is garanteed to be composed of scalars that are commutative
% under multiplication (e.g. reals and complex numbers), f21, f31 and f32 
% do not need to be computed, since they will be equal to f12, f13 and f23
% and the F matrix will be symmetric.
% However, to avoid incorrect results if in the future this function is
% used with vectors built over non-commutative elements (e.g.: vectors of
% quaternions or matrices), I decided to not use this optimization.
