function BB = calc_BlackBody_mags(params, consts, b1, b2,...
    band_order, verbose)
% Compute magnitudes at the surface in bands b1 and b2 for black-bodies
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

if verbose
    disp('Computing magnitudes and color indexes for black-bodies.');
end

% Create an array of black-body temperatures (BB.T) and its multiplicative
% inverse (BB.Tinv).
BB.Tinv = 1/params.Tmax : params.TinvStep : 1/params.Tmin;
BB.T = 1./ BB.Tinv;

% Limits for numerical integration
minlambda = min( min(b1.data(:,1)), min(b2.data(:,1)));
maxlambda = max( max(b1.data(:,1)), max(b2.data(:,1)));
log10from = log10(minlambda) - 2*params.log10integStep;
log10to   = log10(maxlambda) + 2*params.log10integStep;
lambda = 10 .^ (log10from : params.log10integStep : log10to);

% resamples spectral response for bands 1 and 2:
B1 = interp1(b1.data(:,1), b1.data(:,2), lambda, 'pchip', 0);
B2 = interp1(b2.data(:,1), b2.data(:,2), lambda, 'pchip', 0);

% Computes b1 and b2 magnitudes for each black-body temperature in BB.T
BB.b1mag = zeros(size(BB.T));
BB.b2mag = zeros(size(BB.T));
const1 = 2 * pi * consts.h * consts.c^2; % 2*pi*h*c^2
const2 = consts.h * consts.c / consts.k; % h*c/k
for n=1:size(BB.T, 2)
    % Computes spectral exitance of a black-body for wavelengths in lambda
    Tbb = BB.T(n); % black body temperature in Kelvins
    Mbb = const1 ./ lambda.^5 ./ ( exp(const2/Tbb ./ lambda) - 1);

    lambdadif = filter([1 -1], 1, lambda); % obtains d_lambda
    lambdadif(1) = 0;

    % Computes fluxes at B1 and B2 bands emitted from the black body
    B1_fluxint = filter(1, [1 -1], Mbb .* lambdadif .* B1);
    B1flux = B1_fluxint(end);

    B2_fluxint = filter(1, [1 -1], Mbb .* lambdadif .* B2);
    B2flux = B2_fluxint(end);

    % Computes apparent magnitudes for this black-body for an observer at
    % its surface in spectral bands B1 and B2.
    BB.b1mag(n) = 2.5 * log10(b1.Ref / B1flux);
    BB.b2mag(n) = 2.5 * log10(b2.Ref / B2flux);
end

BB.lambda = lambda;
BB.B1 = B1;
BB.B2 = B2;

% computes color index = difference between magnitudes in two different
% spectral bands. Also creates a name for this color index.
if band_order > 0
    BB.color = BB.b1mag - BB.b2mag;
    BB.colorIdxName = [b1.name '-' b2.name];
else
    BB.color = BB.b2mag - BB.b1mag;
    BB.colorIdxName = [b2.name '-' b1.name];
end
BB.band_order = band_order;

end