function mag = calc_mag(lambdavect, bandvect, fluxref,...
    star_lambda, star_flux)
%function mag = calc_mag(lambdavect, bandvect, fluxref,...
%    star_lambda, star_flux)
% Computes stellar magnitude in band defined by bandvect, by
% numerically integrating stellar spectrum given in star_flux
% inputs:
%  lambdavect -> array with wavelength values (must be monotonic)
%  bandvect -> array with normalized band response (must have the same size
%              of 'lambdavect')
%  fluxref -> flux that defines zero point in spectral band defined by
%              'bandvect'
%  starlambda -> array with wavelength values for stellar spectra (must be
%               monotonic)
%  star_flux -> array with stellar spectra (must have the same size
%              'starlambda')
% output:
%  mag -> computed magnitude in band specified by 'bandvect'.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

star_lambdadif = filter([1 -1], 1, star_lambda);
star_lambdadif(1) = 0;

if (min(lambdavect) < min(star_lambda)) || ...
   (max(lambdavect) > max(star_lambda))
    warning(['stellar spectra provided for calc_mag does not cover'...
        ' all wavelength range used in the spectral band definition.'...
        ' Result may be innacurate.']);
end

% Computes stellar flux
bandres = interp1(lambdavect, bandvect, star_lambda, 'pchip', 0);
fluxint = filter(1, [1 -1], star_flux .* bandres .* star_lambdadif);
star_bflux = fluxint(end);
mag = -2.5*log10(star_bflux / fluxref);
end