function outputs = ...
    calc_star_celb(starname, fitsfilename, band1, band2, input_params)
%outputs = 
% calc_star_celb(starname, fitsfilename, band1, band2, input_params);
% Computes the lower bound of centroiding error for a given star by
% numerical integration using the spectra supplied in a given FITS file
% and compare that with the lower bound for a black-body with the same
% magnitude and color index of that star.
% Inputs:
%   starname => star name to be displayed in plots
%   fitsfilename => A FITS file containing stellar spectrum
%   band1, band2 => Strings designating the spectral bands to be used, as
%                   specified in loadBands.m. band1 specifies the primary
%                   band and band2 specifies the secondary band. If band1
%                   is at a longer wavelength than band2, then
%                   input_params.band_order must be set to -1.
%   input_params => optional argument specifying function parameters.
%                   If it is not specified, defaults are used.
% Outputs:
%   outputs <= some internal variables saved to the output.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Input parameters - star tracker diameter and exposure time.
default_params.diam = 1;  % Star tracker diameter in meters
default_params.t = 1;     % Exposure time in seconds
default_params.Tmin = 2000;
default_params.Tmax = 30000;
default_params.TinvStep = 4e-7; % inverse of temperature step when 
  % generating a list of black bodies with temperatures from Tmin to Tmax.
default_params.log10integStep = 0.0001; % base-10 logarithm of integration
  % step in wavelength.
default_params.band_order = 1; % 1 for normal order (primary band at a 
  % shorter  wavelength than the secondary band), and -1 for inverse order
  % (primary band at a longer wavelength than the secondary band).
default_params.lmin = 1e-7;
default_params.lmax = 1e-5;
default_params.verbose = true;
default_params.plotlist = [true, true, true]; % controls plot generation
  % and display.
default_params.plotlistprintpdf = [false, false, false]; % if true,
  % corresponding plot is printed to a .pdf file.
default_params.figpath=''; % place to put output figures.
default_params.plotpos = []; % This can be set to
  % [left, bottom, width, height] to modify the default position of the
  % plot inside the figure or left as an empty matrix to not modify the
  % default plot position. If set to [left, bottom, width, height], it
  % changes the figure 'axis' (or more properly 'axes') position. For help,
  % see Matlab command 'axes' or Octave axes object 'position' property.
default_params.figscreenoffset = [0 0]; % [x, y] screen offset in inches
  % for the figures to be plotted.

if nargin < 5
    params = default_params;
else
    params = merge_input_params(input_params, default_params);
end

verbose = params.verbose;
outputs.params = params; % copies some internal variables to the output.


fprintf('STR diameter = %.6g m   exposure time = %.6g s\n',...
    params.diam, params.t);
fprintf(['Limits for black-body temperature:'...
    '  Tmin = %.6g K  Tmax = %.6g K.\n'], params.Tmin, params.Tmax);

% Load spectral data spectral bands
[blist, Vegamag] = loadBands; %#ok<ASGLU>

% According to parameters band1 and band2 loads data from blist and Vegamag
b1.name = band1;
b1.data = eval(['blist.' band1 ';']);
b1.Vegamag = eval(['Vegamag.' band1 ';']);

b2.name = band2;
b2.data = eval(['blist.' band2 ';']);
b2.Vegamag = eval(['Vegamag.' band2 ';']);

% Compute reference fluxes used as magnitude zero points, using the star
% Vega (alpha Lyr) as a reference:
b1.Ref = VegaRefmag(b1.data, b1.Vegamag);
b2.Ref = VegaRefmag(b2.data, b2.Vegamag);

if verbose
    fprintf('Reference flux for band %s = %e W/m^2\n', b1.name ,b1.Ref);
    fprintf('Reference flux for band %s = %e W/m^2\n', b2.name ,b2.Ref);
end

% loads Physical and Mathematical constants
consts = loadConsts(verbose);
outputs.consts = consts;

% Compute black-body magnitudes. Also returns in struct BB temperatures
% and wavelength tables for interpolation and numerical integration.
% BB is an acronym for black-body.
BB = calc_BlackBody_mags(params, consts, b1, b2,...
    params.band_order, verbose);
outputs.BB = BB;

% Loads and processes spectral data for the star
% The FITS file should be in the same format provided by calibration
% files in the CALSPEC library (follow hyperlink in "calspec/readme.txt"
% for details).
stardata = read_calspec(fitsfilename);
% Multiplication by 1e-10 is to convert from angstroms to meters
star_lambda = stardata{1} * 1e-10;
% Multiplication by 1e7 is to convert from erg/s/cm^2/angstrom to W/m^2/m.
star_flux = stardata{2} * 1e7;
outputs.star_lambda = star_lambda;
outputs.star_flux = star_flux;

% Star magnitude and color index
star_mag1 = calc_mag(b1.data(:,1), b1.data(:,2), b1.Ref,...
    star_lambda, star_flux);
star_mag2 = calc_mag(b2.data(:,1), b2.data(:,2), b2.Ref,...
    star_lambda, star_flux);
outputs.star_mag1 = star_mag1;
outputs.star_mag2 = star_mag2;
[star_T, star_C] = calc_equiv_black_bodies(...
    star_mag1, star_mag2, BB, b2.name, verbose);
outputs.star_T = star_T;
outputs.star_C = star_C;
fprintf('For %s:  %s=%f, %s=%f, %s=%f T=%fK C=%e\n',...
    starname, b1.name, star_mag1, b2.name, star_mag2,...
    BB.colorIdxName, (star_mag1 - star_mag2) * sign(BB.band_order),...
    star_T, star_C);

% Computes the reciprocal of the variance lower bound for a black body
% that is equivalent to the star, using equation (19) in STRLimits.pdf.
ctrInfoFactor = consts.InfoFactor * params.diam^4 * params.t;
star_BB_F = ctrInfoFactor * star_C * star_T^5;
outputs.bb_F = star_BB_F;

fprintf('Lower bound on centr. err. for equiv. black-body = %.16e rad\n',...
    1/sqrt(star_BB_F));


% Numerically integrates energy flux and number of photons for the star
star_lambdadif = filter([1 -1], 1, star_lambda); %differentiates
star_lambdadif(1) = 0;
star_fluxint = filter(1, [1 -1], star_flux .* star_lambdadif); %integrates
starflux = star_fluxint(end);
fprintf('%s flux = %e W/m^2\n', starname, starflux);
star_phflux = star_flux .* star_lambda /(consts.h * consts.c);
star_phfluxint = filter(1, [1 -1], star_phflux .* star_lambdadif); %integrates
starphflux = star_phfluxint(end);
fprintf('%s photon flux = %e photons/m^2/s\n', starname, starphflux);

% Number of photons collected per unit wavelength from the star:
nph_lambda = pi/4 * params.diam^2 * params.t .* star_phflux;

% derivative of centroiding Fisher information for the star, from eq. (17):
star_dF_dlambda = (pi * params.diam ./ star_lambda) .^2 .* nph_lambda;

% Numerically integrating star_dF_dlambda to obtain the Fisher information
% for the star:
star_dFint = filter(1, [1 -1], star_dF_dlambda .* star_lambdadif); %integrates
star_F = star_dFint(end);
outputs.star_F = star_F;

fprintf('Lower bound on centr. err. for %s = %.16e rad\n',...
    starname, 1/sqrt(star_F));


% Numerically integrates equivalent black-body spectrum from 100 pm to 1m.
lambda2 = 10.^(-10:0.00001:0);
Mbb2 = 2*pi*consts.h*consts.c^2 ./ lambda2.^5 ./...
    ( exp(consts.h*consts.c/consts.k/star_T ./ lambda2) - 1);
flux_bb2 = Mbb2 * star_C;
lambda2_dif = filter([1 -1], 1, lambda2); %differentiates
lambda2_dif(1) = 0;
flux_bb2int = filter(1, [1 -1], flux_bb2 .* lambda2_dif); %integrates
fprintf('BB2 flux = %e W/m^2\n', flux_bb2int(end));
phflux_bb2 = flux_bb2 .* lambda2 /consts.h /consts.c;
phfluxint_bb2 = filter(1, [1 -1], phflux_bb2 .* lambda2_dif); %integrates
fprintf('BB2 flux = %e photons/m^2/s\n', phfluxint_bb2(end));
nph_lambda2 = pi/4 * params.diam^2 * params.t .* phflux_bb2;
bb2_dF_dl = (pi * params.diam ./ lambda2) .^2 .* nph_lambda2;
bb2_dFint = filter(1, [1 -1], bb2_dF_dl .* lambda2_dif);
bb2_F = bb2_dFint(end);
outputs.bb2_F = bb2_F;
fprintf(['Lower bound on centr. err. for equiv.'...
    ' black-body = %.16e rad\n'], 1/sqrt(bb2_F));



% % % Figures % % %
nfigures = 3;
figs = zeros(1,nfigures);
yscaletype = 'lin';

% Fig1 - Spectral flux in power density per wavelength interval
if params.plotlist(1)
    figs(1) = figure;
    plot(star_lambda, star_flux * 1e-9, '-');
    hold on;
    plot(lambda2, flux_bb2 * 1e-9, 'k--');
    set (gca,'xscale', 'log');
    set (gca,'yscale', yscaletype);
    ytop = max([star_flux; flux_bb2'])*1e-9 * 1.1;
    axis([params.lmin params.lmax ytop*1e-4 ytop]);
    grid on;
    figposition = [2.2 1 3 3] + [params.figscreenoffset 0 0];
    adjust_figure(figposition, params);
    xlabel('wavelength / m');
    ylabel('spectral flux / [(W/m^2)/nm]');
    %legend(starname, 'Eq. B.B.', 'Location', 'NorthEast');
    title(starname);
end

% Fig2 - Spectral flux in power density per decade
%   For those not familiar with the term:
%        one decade = base 10 logarithmic interval of frequency or
%        wavelength. See for example:
%        https://en.wikipedia.org/wiki/Decade_(log_scale)
if params.plotlist(2)
    figs(2) = figure;
    plot(star_lambda, star_flux .* star_lambda * log(10), '-');
    hold on;
    plot(lambda2, flux_bb2 .* lambda2 * log(10), 'k--');
    set (gca,'xscale', 'log');
    set (gca,'yscale', yscaletype);
    ytop = max([star_flux .* star_lambda; flux_bb2' .* lambda2'])...
        * log(10) * 1.1;
    axis([params.lmin params.lmax ytop*1e-4 ytop]);
    grid on;
    figposition = [5.4 1 3 3] + [params.figscreenoffset 0 0];
    adjust_figure(figposition, params);
    xlabel('wavelength / m');
    ylabel('spectral flux / [(W/m^2)/decade]');
    %legend(starname, 'Eq. B.B.', 'Location', 'NorthEast');
    title(starname);
end

% Fig3 - Spectral flux in power density per frequency interval
if params.plotlist(3)
    figs(3) = figure;
    plot(star_lambda, star_flux .* star_lambda.^2 / consts.c * 1e15, '-');
    hold on;
    plot(lambda2, flux_bb2 .* lambda2.^2 /consts.c * 1e15, 'k--');
    set (gca,'xscale', 'log');
    set (gca,'yscale', yscaletype);
    ytop = max([star_flux .* (star_lambda .^ 2);...
        flux_bb2' .* ((lambda2') .^2) ]) /consts.c * 1e15 * 1.1;
    axis([params.lmin params.lmax ytop*1e-4 ytop]);
    grid on;
    figposition = [8.6 1 3 3] + [params.figscreenoffset 0 0];
    adjust_figure(figposition, params);   
    xlabel('wavelength / m');
    ylabel('spectral flux / [(W/m^2)/PHz]');
    %legend(starname, 'Eq. B.B.', 'Location', 'NorthEast');
    title(starname);
end


% Prints figures
for k=1:nfigures
    if params.plotlist(k)
        figure(figs(k));
                
        % Saves figure to a file
        if params.plotlistprintpdf(k)
            figfilename = sprintf('%s%s%s_fig%d',...
                params.figpath, starname, BB.colorIdxName, k);
            print('-dpdf', figfilename);
        end
    end
end

outputs.figs = figs;
end


% Helper function for adjusting figure size and position
function adjust_figure(figposition, params)
if isempty(params.plotpos) == false % Custom axes position set.
    % Adjusts plot position and dimensions inside the figure.
    set(gca,'Position', params.plotpos);
end
set(gcf,'Units', 'inches');
set(gcf,'Position', figposition);
set(gcf,'PaperSize', [3 3]);
set(gcf,'PaperUnits', 'inches');
set(gcf,'PaperPosition', [0 0 3 3]);
end