% Script that generates the p.d.f. for centroiding assuming that
% centroiding errors when exactly N photons are observed is given by a
% Gaussian distribution.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Parameters:
iota = 10; % mean number of photons (e.g., 1, 10 or 100).
dx = 0.001; % integration step
limx = 15/sqrt(iota); % integration limits


% Computations:
x = -limx:dx:limx;
Nmax = ceil(2*iota)+50;
N = 0:Nmax;

% P = an array where each element P(n+1) gives the probability of
% observing n stars. P follows a Poisson probability mass function.
% Note: original formula: P = iota.^N * exp(-iota) .* 1./factorial(N);
%       the formula below is equivalent and numerically more stable:
P = exp(N*log(iota) - iota - gammaln(N + 1));


% Compute the centroid probability distribution, given that at least one
% photon was observed, assuming that centroiding errors when exactly N
% photons are observed is given by a Gaussian distribution.
% k is the number of observed photons.
Gy = zeros(size(x));
for k=1:Nmax
    % for each k, the standard deviation is 1/sqrt(k)
    gaussian_prob = 1/sqrt(2*pi/k) * exp(-0.5 * x.^2 *k);
    Gy = Gy + P(k+1) * gaussian_prob;
end
Gy = Gy / (1 - exp(-iota)); % Normalizes Gy


% Gx = Gaussian with the same peak value as Gy.
Gg = 1/sqrt(2*pi) * exp(-0.5 * x.^2);
s = max(Gg) / max(Gy);
Gx = 1/sqrt(2*pi*s*s) * exp(-0.5 * x.^2 /s /s);
fprintf('s=%.18f\n', s);


% Numerical check:
fprintf('sum(Gx)*dx - 1 = %e\n', sum(Gx)*dx - 1);
fprintf('sum(Gy)*dx - 1 = %e\n', sum(Gy)*dx - 1);

% Plot
semilogy(x, Gx, '-', x, Gy, 'r-');
legend('Gx', 'Gy');
grid on;

% Relative error estimate
sGx = sqrt(sum(x.^2 .* Gx)*dx); % stddev for Gx
sGy = sqrt(sum(x.^2 .* Gy)*dx); % stddev for Gy
sGz = sqrt(1/iota); % Given by equation 13, normalized by lambda/(4pi*Delta_x).

fprintf('sigma(Gx) = %f\n', sGx);
fprintf('sigma(Gy) = %f\n', sGy);
fprintf('1/sqrt(iota) = %f\n', sGz);
rr = [sGx/sGy sGx/sGz sGy/sGx sGy/sGz sGz/sGx sGz/sGy];
maxerr = max(rr) - 1;
fprintf('max. rel. err. = %f\n', maxerr);

if (P(end) > eps)
    warning('Nmax is too small. Results may be innacurate');
end
