% Script used to generate Figure 7 of the "Theoretical Limits of Star
% Sensor Accuracy" journal article.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Release notes:
%
% v0.4 - 2019-11-24
%   - Upper extrapolation curve (lines 3 and 4) updated.
%   - Included better explanation for upper extrapolation curve.
%
% v0.3 - 2019-11-07:
%   - Replaced tr(P_thetatheta) with theta_{rms} = sqrt(tr(P_thetatheta))
%     in the vertical axis.
%   - Improved y-axis label generation in Matlab.
%
% v0.2 - 2019-09-25:
%   - Replaced 1/tr(P_thetatheta) with tr(P_thetatheta) in the vertical
%     axis, thus reversing the vertical axis.
%   - Replaced calls to function 'plot' with cals to 'loglog'.
%   - Improved plot generation in Matlab and in GNU Octave.
%   - Other minor edits.
%
% v0.1 - 2019-09-18:
%   - First public release.

clear;
close all;

nsrc = 3; % number of sources

% Data files with outputs to be plotted.
datafile{1} = 'outputs/HippBVI_out.mat';
datafile{2} = 'outputs/Tycho2_out.mat';
datafile{3} = 'outputs/2MASS_out.mat';

% Scripts used to generate data files if they are not present
datascript{1} = 'calc_HipBVI.m';
datascript{2} = 'calc_Tycho2.m';
datascript{3} = 'calc_2MASS.m';

% Flags indicating the presence of data files
dataf = zeros(1, nsrc);

% Loads data files if present
for k=1:nsrc
    if exist(datafile{k},'file')
        dataf(k) = true;
        load(datafile{k});
    else
        warning('File %s not found. Run script %s to generate it.', ...
            datafile{k}, datascript{k});
    end
end

if dataf(1)
    Covtrace_BV = calc_covtrace(outputs_BV);
    Covtrace_VI = calc_covtrace(outputs_VI);
end

if dataf(2)
    Covtrace_Tyc = calc_covtrace(outputs_Tyc2);
end

if dataf(3)
    Covtrace_JH = calc_covtrace(outputs_JH);
    Covtrace_JK = calc_covtrace(outputs_JK);
    Covtrace_HK = calc_covtrace(outputs_HK);
end


if sum(dataf) == 0
    error(['At least one data file must be present. '...
        'See warning messages above.']);
end


% Generates figure
fig = figure;
leglist = {}; % list of items to show in the legend

if dataf(1)
    loglog (outputs_BV.Nlist, sqrt(Covtrace_BV), 'bs-');
    hold on;
    leglist{end+1} = 'Hip. B-V';
    loglog (outputs_VI.Nlist, sqrt(Covtrace_VI), 'r^-');
    leglist{end+1} = 'Hip. V-I';
end
if dataf(2)
    loglog (outputs_Tyc2.Nlist, sqrt(Covtrace_Tyc), 'bh-');
    hold on;
    leglist{end+1} = 'Tyc2 B_T-V_T';
end
if dataf(3)
    loglog (outputs_JH.Nlist, sqrt(Covtrace_JH), 'bo-');
    hold on;
    leglist{end+1} = '2MASS J-H';
    loglog (outputs_JK.Nlist, sqrt(Covtrace_JK), 'k*-');
    leglist{end+1} = '2MASS J-K_s';
    loglog (outputs_HK.Nlist, sqrt(Covtrace_HK), 'rd-');
    leglist{end+1} = '2MASS H-K_s';
end

grid on;

% Adjusts figure size.
set(gcf,'Units', 'inches');
set(gcf,'Position', [1 1 6 5]);
set(gcf,'PaperSize', [6 5]);
set(gcf,'PaperUnits', 'inches');
set(gcf,'PaperPosition', [0 0 6 5]);
if isoctave()
    legend(leglist, 'location', 'northeast');
    set(gca,'Position',[0.14 0.11 0.83 0.86]);
    %topbot = [1.2e-29 9e-23];
    topbot = [1.2e-15 9e-12];
else
    legend(leglist, 'location', [0.6 0.65 0.2 0.2]);
    set(gca,'Position',[0.11 0.1 0.86 0.87]);
    topbot = [5.5e-15 4e-12];
    axis([1 1e12 topbot]);
end 


% Axes labels
xlabel('Number of stars');

if isoctave() % vartheta missing in Octave 5.1.0 .PDF print driver
    ylabel('\theta_{rms} [rad]   for  D = 1 m  and  t = 1 s');
else
    % % Option 1
    % % Using default font, but not possible to add bar over vartheta:
    % ylabel('\vartheta_{rms}  [rad]   for  D = 1m  and  t = 1s');
    
    % Option 2:
    % Using Latex interpreter, but this changes the font!
    ylabel(['$\bar\vartheta_{rms}$ [rad]\, for\,'...
         '$D$ = 1\,m\, and\, $t$ = 1\,s'], 'interpreter', 'latex', ...
         'FontSize', 13);
end


% line1 = marks 300 billion stars in the Milky Way
line1 = line([3e11 3e11], topbot, ...
    'LineStyle', '-.', 'Color', 'k');

% lower extrapolation curve:
% line2 = rough extrapolation from Hipparcos and 2MASS with H and K bands.
line2 = line([2.5e2 5e11],[5.59e-14 1.155e-14], ...
    'LineStyle', '--', 'Color', 'k');

% upper extrapolation curve, lines 3 and 4:
% line3 = rough extrapolation from Tycho-2
line3 = line([1e6 5e8],[7.5e-14 5.8e-14], ...
    'LineStyle', '--', 'Color', 'k');

% line4 = continuation of line4
line4 = line([5e8 5e11],[5.8e-14 5.8e-14], ...
    'LineStyle', '--', 'Color', 'k');

% Notice about the shape of the upper extrapolation curve (lines 3 and 4):
% The upper extrapolation curve (lines 3 and 4) was drawn parallel to the
% curves generated by the 2MASS data. Starting at 5e8 stars (roughly the
% end of 2MASS star catalog), conservatively, we make it flat, because as
% the number of stars considered increase, there is a tendency that the
% contribution to the attitude information matrix of each additional
% logarithmic interval of stars (e.g, an 'octave' or a 'decade' of stars
% when stars are sorted by brightness) will decrease, as we start picking
% stars farther away. Stars that are very far from Earth (many kiloparsecs)
% have a greater tendency to be concentrated in the same side of the sky
% than nearby stars, usually being +- 45 degrees from the direction of the
% center of our galaxy, as many of them will be stars from the other side
% of the galaxy. When most attitude references are located in the same
% spot/direction in the sky, the rotation angle around the center of that
% spot becomes less observable, limiting attitude knowledge.

% Markers
plot(3e11, 1.2e-14, 'r.', 'MarkerSize', 20);
plot(3e11, 5.8e-14, 'r.', 'MarkerSize', 20);

% Arrows from markers
line5 = line([3e11 3e11], [1.2e-14 7e-15], 'Color', 'r');
line6 = line([2.5e11 3e11 3.6e11], [8.165e-15 7e-15 8.165e-15], ...
        'Color', 'r');
line7 = line([2e11 4.5e11], [7e-15 7e-15], ...
        'Color', 'r', 'LineWidth', 2);

line8  = line([3e11 3e11], [5.59e-14 1e-13], 'Color', 'r');
line9  = line([2.5e11 3e11 3.6e11], [8.95e-14 1e-13 8.95e-14], 'Color', 'r');
line10 = line([2e11 4.5e11], [1e-13 1e-13], 'Color', 'r', 'LineWidth', 2);

% Prints figure to PDF file
print(gcf, 'outputs/extrapolation','-dpdf');
