% Script used to generate figures 3, 4 and 5 in the paper
% By Marcio A. A. Fialho - date: 2019-05-24 (yyyy-mm-dd).
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

clear;
close all;

% Load spectral data for Johnson's UBV filters, as given in
%        M. Bessell and S. Murphy. "Spectrophotometric Libraries, Revised
%        Photonic Passbands, and Zero Points for UBVRI, Hipparcos, and
%        Tycho Photometry." Publications of the Astronomical
%        Society of the Pacific. 124: 140-157, February 2012.
Bessell2012;

% Convert normalized system photon response functions (S(lambda)) loaded
% from Bessell2012.m to normalized system energy response functions
% (S'(lambda)), according to equation A9 in (Bessell & Murphy, 2012,
% page 154):

bandV_un = bandq_V(:,1) .* bandq_V(:,2); % unnormalized energy response
bandB_un = bandq_B(:,1) .* bandq_B(:,2); % unnormalized energy response
band_V = [bandq_V(:,1), bandV_un / max(bandV_un)];
band_B = [bandq_B(:,1), bandB_un / max(bandB_un)];

% Compute reference fluxes used as magnitude zero points, using the star
% Vega (alpha Lyr) as a reference:
Bref = Refmagx_Vega(band_B, 0.029 ,'B');
Vref = Refmagx_Vega(band_V, 0.030 ,'V');

% Loads Mathematical and Physical constants
consts = loadConsts(false);
c = consts.c;
h = consts.h;
kb = consts.k;
sigmaSB = consts.sigmaSB; % Stefan-Boltzmann constant
fprintf('sigmaSB = %.8g W/m^2/K^4\n', sigmaSB);
tauSB = consts.tauSB; % Equivalent of 'Stefan-Boltzmann' constant for
                      % number of photons.
fprintf('tauSB = %.8g photons/s/m^2/K^3\n', tauSB);

% Wavelength ranges and temperature ranges for numerical integration
disp('Computing magnitudes and color indexes for black-bodies');
tic;
lambda = 10.^(-6.5:0.0001:-6.1);
T = 1./ (1/250000:4e-7:1/1800);
Bmag = zeros(size(T));
Vmag = zeros(size(T));

% Computes B and V for black bodies with temperatures from T(1) to T(end)
for n=1:size(T,2)
    % Computes spectral exitance of a black-body for wavelengths in lambda
    Tbb = T(n); % black body temperature in Kelvins
    Mbb = 2*pi*h*c^2 ./ lambda.^5 ./ ( exp(h*c/kb/Tbb ./ lambda) - 1);

    lambdadif = filter([1 -1], 1, lambda); % obtains d_lambda
    lambdadif(1) = 0;

    % Computes fluxes at B and V band emitted from the black body
    % Multiplication by 1e-10 is to convert from Angstroms to meters.
    B = interp1(band_B(:,1)*1e-10, band_B(:,2), lambda, 'pchip', 0);
    B_fluxint = filter(1, [1 -1], Mbb .* lambdadif .* B);
    Bflux = B_fluxint(end);
    V = interp1(band_V(:,1)*1e-10, band_V(:,2), lambda, 'pchip', 0);
    V_fluxint = filter(1, [1 -1], Mbb .* lambdadif .* V);
    Vflux = V_fluxint(end);

    % Computes B and V absolute magnitudes for this black-body:
    % Multiplication by 1e-3 is to convert from erg/s/cm^2 to W/m^2.
    Bmag(n) = 2.5*log10(Bref*1e-3 / Bflux);
    Vmag(n) = 2.5*log10(Vref*1e-3 / Vflux);
end
BV = Bmag - Vmag;
Tinv = 1 ./ T;
toc;



% % % Figures % % %
display('Plotting figures');
tic;

% Figure property names that may differ between Octave and Matlab:
%
%    Notice: Tests have shown that property names below defined for Octave
%            also work with Matlab R2015b. However this code uses the
%            documented property names for Matlab instead, to avoid
%            possible problems with future Matlab releases.
%
if isoctave % GNU-Octave 5.1.0 (and also Matlab R2015b, but not guaranteed)
    GridAlpha = 'gridalpha';
    MinorGridAlpha = 'minorgridalpha';
    GridLine = 'gridlinestyle';
    MinorGridLine = 'minorgridlinestyle';
else % Matlab R2015b (using recommended property names)
    GridAlpha = 'GridAlpha';
    MinorGridAlpha = 'MinorGridAlpha';
    GridLine = 'GridLine';
    MinorGridLine = 'MinorGridLine';
end


% B and V passband responses:
fig1 = figure;
plot(lambda*1e9, B, 'b--');
hold on;
plot(lambda*1e9, V, 'k-');
axis([335 715 0 1]);
set(gcf,'Units','inches');
figsize=[4 3]; % figure width and height in inches.
set(gcf,'Position', [2, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
grid on;
grid minor;
set (gca, MinorGridAlpha, get(gca, GridAlpha)*0.25);
set (gca, MinorGridLine, get(gca, GridLine));
xlabel('wavelength (nm)');
ylabel('normalized energy response');
title('B and V passbands');
print(gcf, 'outputs/BVresponse', '-dpdf');  % save figure to file


% This figure is used to visually evaluate the accuracy of the numerical
% integration performed.
fig2 = figure;
sfig2 = zeros(1,7);
sfig2(1) = subplot(2,4,1);
semilogx(T, Vmag);
xlabel('Teff / K');
ylabel('V magnitude');
grid on;

sfig2(2) = subplot(2,4,2);
semilogx(T, BV);
xlabel('Teff / K');
ylabel('B-V color index');
grid on;

sfig2(3) = subplot(2,4,3);
semilogx(1 ./ T, Vmag);
xlabel('1/Teff / 1/K');
ylabel('V magnitude');
grid on;

sfig2(4) = subplot(2,4,4);
semilogx(1 ./ T, BV);
xlabel('1/Teff / 1/K');
ylabel('B-V color index');
grid on;

Bdif = filter([1 -1], 1, Bmag);
Bdif(1) = NaN;

Vdif = filter([1 -1], 1, Vmag);
Vdif(1) = NaN;

BVdif = filter([1 -1], 1, BV);
BVdif(1) = NaN;

sfig2(5) = subplot(2,4,5);
semilogx(T, Bdif, '.'); hold on; semilogx(T, Vdif, 'r.');
semilogx(T, BVdif, 'k.'); xlabel('T (K)'); grid on;

sfig2(6) = subplot(2,4,6);
semilogx(1./T, Bdif, '.'); hold on; semilogx(1./T, Vdif, 'r.');
semilogx(1./T, BVdif, 'k.'); xlabel('1/T (1/K)'); grid on;

nidx = size(T, 2); fprintf('nidx = %d\n', nidx);
idx = 1:nidx;
sfig2(7) = subplot(2,4,7);
plot(idx, Bdif, '.'); hold on; plot(idx, Vdif, 'r.');
plot(idx, BVdif, 'k.'); xlabel('index'); grid on;

fprintf('max(abs(Bdif))=%f  max(abs(Vdif))=%f  max(abs(BVdif)=%f\n',...
    max(abs(Bdif)), max(abs(Vdif)), max(abs(BVdif)));


% B and V apparent magnitudes for black-bodies, for an observer located
% at their surface
fig3 = figure;
semilogx(T, Bmag, 'b--');
hold on;
semilogx(T, Vmag, 'k-');
axis([1e3 3e5 -47 -24 ]);
set(gca,'Ydir','reverse')
set(gcf,'Units','inches');
figsize=[4 3];
set(gcf,'Position', [5, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
if isoctave() % Octave 5.1.0
    % legend location 'best' still not implemented in Octave 5.1.0
    legend('B magnitude', 'V magnitude','location','east');
else % Matlab R2015b
    legend('B magnitude', 'V magnitude','location','best');
end
grid on;
grid minor;
xlabel('Temperature (K)');
ylabel('black body magnitudes');
set (gca, MinorGridAlpha, get(gca, GridAlpha)*0.25);
set (gca, MinorGridLine, get(gca, GridLine));
print(gcf, 'outputs/BVmags', '-dpdf');  % save figure to file


% Relation between B-V color index and temperature
fig4 = figure;
semilogx(T, BV, 'k-');
axis([1e3 3e5 -0.55 3.2]);
set(gcf,'Units','inches');
figsize=[3 3];
set(gcf,'Position', [5, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
xlabel('Temperature (K)');
ylabel('B-V color index');
grid on;
grid minor;
set (gca, MinorGridAlpha, get(gca, GridAlpha)*0.25);
set (gca, MinorGridLine, get(gca, GridLine));
%print(gcf, 'outputs/BVtemp', '-depsc2');  % save figure to file
%print(gcf, 'outputs/BVtemp', '-dmeta');  % save figure to file
print(gcf, 'outputs/BVtemp', '-dpdf');  % save figure to file


% Relation between B-V color index and temperature reciprocal
fig5 = figure;
plot(1e6./T, BV, 'k-');
axis([0 600 -0.55 3.2]);
set(gcf,'Units','inches');
figsize=[3 3];
set(gcf,'Position', [5, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
xlabel('1 / Temperature (MK^{-1})');
ylabel('B-V color index');
grid on;
grid minor;
set (gca, MinorGridAlpha, get(gca, GridAlpha)*0.25);
set (gca, MinorGridLine, get(gca, GridLine));
%print(gcf, 'outputs/BVtempinv', '-depsc2');  % save figure to file
%print(gcf, 'outputs/BVtempinv', '-dmeta');  % save figure to file
print(gcf, 'outputs/BVtempinv', '-dpdf');  % save figure to file


toc;
display('Done.');