function data = read_calspec_int(filename)
% Helper function used to read some CALSPEC files in systems without the
% fitsread function. Opens only some .fits files needed by the 
% "THEORETICAL LIMITS OF STAR SENSOR ACCURACY" journal article.
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Written by Marcio A. A. Fialho..

if exist(filename, 'file') == 0
    error('%s: file not found!', filename);
end

[~, fname, ~] = fileparts(filename);

if strcmpi(fname, 'alpha_lyr_stis_008')
    data = read_Vega(filename); % in this file
elseif strcmpi(fname, '1757132_stis_004')
    data = read_1757132(filename); % in this file
elseif strcmpi(fname, 'sun_reference_stis_002')
    data = read_Sun(filename); % in this file
elseif strcmpi(fname, 'kf06t2_stisnic_004')
    data = read_kf06t2(filename); % in this file
elseif strcmpi(fname, 'vb8_stisnic_006')
    data = read_vb8(filename); % in this file
else
    error(['This function supports only some ' ...
        'specific .fits files. File ' filename ...
        ' is not supported.']);
end
end



% alpha_lyr_stis_008.fits
% Custom function for reading 'alpha_lyr_stis_008.fits' with 276,480 bytes,
% containing spectrum for star Vega (Alpha Lyr).
function data = read_Vega(filename)
% Get file size in bytes, must be 276,480 bytes
s = dir(filename);
if s(1).bytes ~= 276480
    error('File %s should have 276480 bytes. Found %d bytes.', ...
        filename, s.bytes);
end

fid = fopen(filename, 'r', 'b'); % opens file in big-endian format

% Reads file header and checks some fields in it.
%  Notice on convention adopted in error messages: first byte is at file
%  offset 0 (Matlab/Octave array/vector index 1)
fheader = fread(fid, 11520); % 11520 bytes.
if strcmp(char(fheader(1:6)'),'SIMPLE') == 0
    error('%s: expected string ''SIMPLE'' at file offset 0', filename);
end
if strcmp(char(fheader(6001:6006)'),'NAXIS1') == 0
    error('%s: expected string ''NAXIS1'' at file offset 6000', filename);
end
if strcmp(char(fheader(6028:6032)'),' 30 /') == 0
    error('%s: expected string '' 30 /'' at file offset 6027', filename);
end
if strcmp(char(fheader(6081:6086)'),'NAXIS2') == 0
    error('%s: expected string ''NAXIS2'' at file offset 6080', filename);
end
if strcmp(char(fheader(6106:6112)'),' 8827 /') == 0
    error('%s: expected string '' 8827 /'' at file offset 6105', filename);
end
if strcmp(char(fheader(8881:8883)'),'END') == 0
    error('%s: expected string ''END'' at file offset 8880', filename);
end
if fheader(end) ~= 32 % 32 is the ASCII code of the SPACE character
    error('%s: Expected a SPACE character at file offset 11519', filename);
end

% Preallocates memory
%  alpha_lyr_stis_008.fits has 8827 entries.
%  Field descriptions from alpha_lyr_stis_008.fits header.
data{1} = zeros(8827, 1);  % Wavelength in angstroms
data{2} = zeros(8827, 1);  % Flux
data{3} = zeros(8827, 1);  % FLAM = Statistical flux error
data{4} = zeros(8827, 1);  % Systematic flux error=0.01*FLAM
data{5} = zeros(8827, 1);  % FWHM spectral resolution
data{6} = zeros(8827, 1);  % Data quality: 1=good, 0=bad 
data{7} = zeros(8827, 1);  % Total exposure time in seconds

% Reads data
%  For help, open the first 11520 bytes of alpha_lyr_stis_008.fits in any
%  text editor configured to open text files with Unix line endings.
for k=1:8827
    data{1}(k) = fread(fid, 1, 'real*8');
    data{2}(k) = fread(fid, 1, 'real*4');
    data{3}(k) = fread(fid, 1, 'real*4');
    data{4}(k) = fread(fid, 1, 'real*4');
    data{5}(k) = fread(fid, 1, 'real*4');
    data{6}(k) = fread(fid, 1, 'integer*2');
    data{7}(k) = fread(fid, 1, 'real*4');
end

% Checks some values for consistency
readdata = [data{1}(1), data{1}(end), data{2}(1), data{2}(end)];
checkdata =  [900.4520263671875, 2999537, ...
    1.2490576969241589989e-17, 1.2576233020936316257e-19];

if isequal(readdata, checkdata) == false
    error(['%s: Data mismatch. ' ...
        '\nExpected = [%.20g, %.20g, %.20g, %.20g]. '...
        '\nFound    = [%.20g, %.20g, %.20g, %.20g]'], ...
        filename, checkdata, readdata);
end
end



% 1757132_stis_004.fits
% Custom function for reading '1757132_stis_004.fits' with 224,640 bytes,
% containing spectrum for star 1757132.
function data = read_1757132(filename)
% Get file size in bytes, must be 224,640 bytes
s = dir(filename);
if s(1).bytes ~= 224640
    error('File %s should have 224,640 bytes. Found %d bytes.', ...
        filename, s.bytes);
end

fid = fopen(filename, 'r', 'b'); % opens file in big-endian format

% Reads file header and checks some fields in it.
%  Notice on convention adopted in error messages: first byte is at file
%  offset 0 (Matlab/Octave array/vector index 1)
headerlen = 11520; % 11520 bytes.
fheader = fread(fid, headerlen);
check_header_entry(fheader, filename, 'SIMPLE', 0);
check_header_entry(fheader, filename, 'NAXIS1', 6000);
check_header_entry(fheader, filename, ' 30 /', 6027);
check_header_entry(fheader, filename, 'NAXIS2', 6080);
check_header_entry(fheader, filename, ' 7097 /', 6105);
check_header_entry(fheader, filename, 'END', 8880);
check_header_lastbyte(fheader, filename);

% Preallocates memory
%  1757132_stis_004.fits has 7097 entries.
%  Field descriptions from 1757132_stis_004.fits header.
n = 7097;
data{1} = zeros(n, 1);  % Wavelength in angstroms
data{2} = zeros(n, 1);  % Flux = Absolutely calibrated net spectrum
data{3} = zeros(n, 1);  % FLAM = Statistical flux error
data{4} = zeros(n, 1);  % Systematic flux error=0.01*FLAM
data{5} = zeros(n, 1);  % FWHM spectral resolution
data{6} = zeros(n, 1);  % Data quality: 1=good, 0=bad 
data{7} = zeros(n, 1);  % Total exposure time in seconds

% Reads data
%  For help, open the first 11520 bytes of 1757132_stis_004.fits in any
%  text editor configured to open text files with Unix line endings.
for k=1:n
    data{1}(k) = fread(fid, 1, 'real*8');
    data{2}(k) = fread(fid, 1, 'real*4');
    data{3}(k) = fread(fid, 1, 'real*4');
    data{4}(k) = fread(fid, 1, 'real*4');
    data{5}(k) = fread(fid, 1, 'real*4');
    data{6}(k) = fread(fid, 1, 'integer*2');
    data{7}(k) = fread(fid, 1, 'real*4');
end

% Checks some values for consistency
readdata = [data{1}(1), data{1}(end), data{2}(1), data{2}(end)];
checkdata = [1141.1171875, 319894.90625, ...
    1.3108000294882808353e-16, 2.4364601711575162378e-20];

if isequal(readdata, checkdata) == false
    error(['%s: Data mismatch. ' ...
        '\nExpected = [%.20g, %.20g, %.20g, %.20g]. '...
        '\nFound    = [%.20g, %.20g, %.20g, %.20g]'], ...
        filename, checkdata, readdata);
end
end



% sun_reference_stis_002.fits
% Custom function for reading 'sun_reference_stis_002.fits' with 37,440
% bytes, containing the spectrum for the Sun.
function data = read_Sun(filename)
% Get file size in bytes, must be 37,440 bytes
s = dir(filename);
if s(1).bytes ~= 37440
    error('File %s should have 37440 bytes. Found %d bytes.', ...
        filename, s.bytes);
end

fid = fopen(filename, 'r', 'b'); % opens file in big-endian format

% Reads file header and checks some fields in it.
%  Notice on convention adopted in error messages: first byte is at file
%  offset 0 (Matlab/Octave array/vector index 1)
headerlen = 5760; % 5760 bytes.
fheader = fread(fid, headerlen);
check_header_entry(fheader, filename, 'SIMPLE', 0);
check_header_entry(fheader, filename, 'NAXIS1', 3120);
check_header_entry(fheader, filename, ' 20 /', 3147);
check_header_entry(fheader, filename, 'NAXIS2', 3200);
check_header_entry(fheader, filename, ' 1467 /', 3225);
check_header_entry(fheader, filename, 'END', 5360);
check_header_lastbyte(fheader, filename);

% Preallocates memory
%  sun_reference_stis_002.fits has 1467 entries.
%  Field descriptions from sun_reference_stis_002.fits header.
n = 1467;
data{1} = zeros(n, 1);  % Wavelength in angstroms
data{2} = zeros(n, 1);  % Flux
data{3} = zeros(n, 1);  % SYSERROR
data{4} = zeros(n, 1);  % FWHM
data{5} = zeros(n, 1);  % DATAQUAL

% Reads data
%  For help, open the first 5760 bytes of sun_reference_stis_002.fits in
%  any text editor configured to open text files with Unix line endings.
for k=1:n
    data{1}(k) = fread(fid, 1, 'real*4');
    data{2}(k) = fread(fid, 1, 'real*4');
    data{3}(k) = fread(fid, 1, 'real*4');
    data{4}(k) = fread(fid, 1, 'real*4');
    data{5}(k) = fread(fid, 1, 'real*4');
end

% Checks some values for consistency
readdata = [data{1}(1), data{1}(end), data{2}(1), data{2}(end)];
checkdata = [1195, 26957.3515625, ...
    0.0088615007698535919189, 3.8581380844116210938];

if isequal(readdata, checkdata) == false
    error(['%s: Data mismatch. ' ...
        '\nExpected = [%.20g, %.20g, %.20g, %.20g]. '...
        '\nFound    = [%.20g, %.20g, %.20g, %.20g]'], ...
        filename, checkdata, readdata);
end
end



% kf06t2_stisnic_004.fits
% Custom function for reading 'kf06t2_stisnic_004.fits' with 158,400 bytes,
% containing spectrum for star kf06t2.
function data = read_kf06t2(filename)
% Get file size in bytes, must be 158,400 bytes
s = dir(filename);
if s(1).bytes ~= 158400
    error('File %s should have 158,400 bytes. Found %d bytes.', ...
        filename, s.bytes);
end

fid = fopen(filename, 'r', 'b'); % opens file in big-endian format

% Reads file header and checks some fields in it.
%  Notice on convention adopted in error messages: first byte is at file
%  offset 0 (Matlab/Octave array/vector index 1)
headerlen = 11520; % 11520 bytes.
fheader = fread(fid, headerlen);
check_header_entry(fheader, filename, 'SIMPLE', 0);
check_header_entry(fheader, filename, 'NAXIS1', 6000);
check_header_entry(fheader, filename, ' 30 /', 6027);
check_header_entry(fheader, filename, 'NAXIS2', 6080);
check_header_entry(fheader, filename, ' 4834 /', 6105);
check_header_entry(fheader, filename, 'END', 8880);
check_header_lastbyte(fheader, filename);

% Preallocates memory
%  kf06t2_stisnic_004.fits has 4834 entries.
%  Field descriptions from kf06t2_stisnic_004.fits header.
n = 4834;
data{1} = zeros(n, 1);  % Wavelength in angstroms
data{2} = zeros(n, 1);  % Flux = Absolutely calibrated net spectrum
data{3} = zeros(n, 1);  % FLAM = Statistical flux error
data{4} = zeros(n, 1);  % Systematic flux error=0.01*FLAM
data{5} = zeros(n, 1);  % FWHM spectral resolution
data{6} = zeros(n, 1);  % Data quality: 1=good, 0=bad 
data{7} = zeros(n, 1);  % Total exposure time in seconds

% Reads data
%  For help, open the first 11520 bytes of kf06t2_stisnic_004.fits in any
%  text editor configured to open text files with Unix line endings.
for k=1:n
    data{1}(k) = fread(fid, 1, 'real*8');
    data{2}(k) = fread(fid, 1, 'real*4');
    data{3}(k) = fread(fid, 1, 'real*4');
    data{4}(k) = fread(fid, 1, 'real*4');
    data{5}(k) = fread(fid, 1, 'real*4');
    data{6}(k) = fread(fid, 1, 'integer*2');
    data{7}(k) = fread(fid, 1, 'real*4');
end

% Checks some values for consistency
readdata = [data{1}(1), data{1}(end), data{2}(1), data{2}(end)];
checkdata = [2900.689697265625, 319894.90625, ...
    -7.7565999235496246693e-16, 2.7994435029345504864e-20];

if isequal(readdata, checkdata) == false
    error(['%s: Data mismatch. ' ...
        '\nExpected = [%.20g, %.20g, %.20g, %.20g]. '...
        '\nFound    = [%.20g, %.20g, %.20g, %.20g]'], ...
        filename, checkdata, readdata);
end
end



% vb8_stisnic_006.fits
% Custom function for reading 'vb8_stisnic_006.fits' with 80,640 bytes,
% containing spectrum for star kf06t2.
function data = read_vb8(filename)
% Get file size in bytes, must be 80,640 bytes
s = dir(filename);
if s(1).bytes ~= 80640
    error('File %s should have 80,640 bytes. Found %d bytes.', ...
        filename, s.bytes);
end

fid = fopen(filename, 'r', 'b'); % opens file in big-endian format

% Reads file header and checks some fields in it.
%  Notice on convention adopted in error messages: first byte is at file
%  offset 0 (Matlab/Octave array/vector index 1)
headerlen = 11520; % 11520 bytes.
fheader = fread(fid, headerlen);
check_header_entry(fheader, filename, 'SIMPLE', 0);
check_header_entry(fheader, filename, 'NAXIS1', 6000);
check_header_entry(fheader, filename, ' 30 /', 6027);
check_header_entry(fheader, filename, 'NAXIS2', 6080);
check_header_entry(fheader, filename, ' 2236 /', 6105);
check_header_entry(fheader, filename, 'END', 8880);
check_header_lastbyte(fheader, filename);

% Preallocates memory
%  vb8_stisnic_006.fits has 2236 entries.
%  Field descriptions from vb8_stisnic_006.fits header.
n = 2236;
data{1} = zeros(n, 1);  % Wavelength in angstroms
data{2} = zeros(n, 1);  % Flux = Absolutely calibrated net spectrum
data{3} = zeros(n, 1);  % FLAM = Statistical flux error
data{4} = zeros(n, 1);  % Systematic flux error=0.01*FLAM
data{5} = zeros(n, 1);  % FWHM spectral resolution
data{6} = zeros(n, 1);  % Data quality: 1=good, 0=bad 
data{7} = zeros(n, 1);  % Total exposure time in seconds

% Reads data
%  For help, open the first 11520 bytes of vb8_stisnic_006.fits in any
%  text editor configured to open text files with Unix line endings.
for k=1:n
    data{1}(k) = fread(fid, 1, 'real*8');
    data{2}(k) = fread(fid, 1, 'real*4');
    data{3}(k) = fread(fid, 1, 'real*4');
    data{4}(k) = fread(fid, 1, 'real*4');
    data{5}(k) = fread(fid, 1, 'real*4');
    data{6}(k) = fread(fid, 1, 'integer*2');
    data{7}(k) = fread(fid, 1, 'real*4');
end

% Checks some values for consistency
readdata = [data{1}(1), data{1}(end), data{2}(1), data{2}(end)];
checkdata = [2900.329833984375, 24977.181000000000495, ...
    -3.0569999283976568433e-15, 5.9283001650875435246e-15];

if isequal(readdata, checkdata) == false
    error(['%s: Data mismatch. ' ...
        '\nExpected = [%.20g, %.20g, %.20g, %.20g]. '...
        '\nFound    = [%.20g, %.20g, %.20g, %.20g]'], ...
        filename, checkdata, readdata);
end
end



% Helper function for checking some header entries
function check_header_entry(fheader, filename, entryname, fileoffset)
entrysize = size(entryname, 2);
readstr = fheader(fileoffset + 1 : fileoffset + entrysize);
if strcmp(char(readstr'), entryname) == 0
    error('%s: expected string ''%s'' at file offset %d',...
        filename, entryname, fileoffset);
end
end

% Helper function for checking header
function check_header_lastbyte(fheader, filename)
if fheader(end) ~= 32 % 32 is the ASCII code of the SPACE character
    error('%s: Expected a SPACE character at file offset %d', ...
        filename, size(fheader, 1) - 1);
end
end
